/*
	Programa: xadrez.exe
    Descrio: header da biblioteca xeque, onde encontram-se as declaraes de funes, defines e informaes da bilioteca.
    Programador: Eduardo Prates Albuquerque
    RGM: 21500
    Data da ltima modificao: 21/09/2016
*/

#ifndef _ARVORE_H_
	#define _ARVORE__1

	/*Lista ligada de jogadas possveis*/
    typedef struct reg {
    	int conteudo;
    	struct reg *prox;
    }Jogada;

    /*Arvore de jogadas*/
    typedef struct no {
      	int tabuleiro[8][8];
    	int TipoNo,jogada,nivel_arvore,fim_jogo;
    	float avaliacao;
    	struct no *f1, *f2, *f3, *f4, *f5, *f6, *f7, *f8, *f9, *f10, *f11, *f12, *f13, *f14, *f15, *f16, *f17, *f18, *f19, *f20;
    }Arvore;

    /***************************************************************************************************************************
	  Arvore* cria_no ();
	  Descrio:
		- Cria um n dentro da rvore de jogadas.
		- Primeiro parmentro do tipo "Arvore" recebe o estado do jogo.
		- Segundo parmentro do tipo  "int" recebe qual jogador realiza a jogada.
		- Terceiro parmentro do tipo "int" recebe a matriz das posies do tabuleiro.
		- Quarto parmentro do tipo "int" recebe a posio atual da pea e pra onde deseja-se mov-la.
		- Quinto parmentro do tipo "int" recebe o nvel de profundidade da rvore de jogadas.
		- Retorna a posio da movimento realizado
	*****************************************************************************************************************************/
    void cria_no (struct no **, int,int M[][8],int, int);

    /***************************************************************************************************************************
	  void aloca ();
	  Descrio:
		- Aloca as jogadas possveis em um determinado n filho da rvore.
		- Primeiro parmentro do tipo "Arvore" recebe o estado do jogo.
		- Segundo parmentro do tipo  "Lista" recebe as possibilidades de jogada a serem feitas.
		- Terceiro parmentro do tipo "int" recebe qual jogador deve realizar a prxima jogadas.
		- Quarto parmentro do tipo "int" recebe a Matriz com a atual posio das peas.
		- Quinto parmentro do tipo "int" recebe o nvel de profundidade da rvore de jogadas.
		- No H retorno.
	*****************************************************************************************************************************/
    void aloca (struct no *,struct reg *, int,int M[][8], int);
    /***************************************************************************************************************************
	  struct no* Liberar ();
	  Descrio:
		- Libera memria da rvore de jogadas.
		- Primeiro parmentro do tipo "Arvore" recebe o estado do jogo.
		- No h retorno.
	*****************************************************************************************************************************/
    struct no* Liberar(struct no **); /*Libera a arvore completa*/

    /***************************************************************************************************************************
	  Void Imprimir ();
	  Descrio:
		- Percorre a rvore de jogadas, para que cada n seja analisado.
		- Primeiro parmentro do tipo "Arvore" recebe a arvore de jogadas.
		- Segundo parmentro do tipo "int" recebe a matriz das posies do tabuleiro.
        - Terceiro parmentro do tipo "int" recebe a posicao atual do rei branco.
        - Quarto parmentro do tipo "int" recebe a posicao atual do rei preto.
		- Quinto parmetro do tipo "int" recebe a quantidade de lances realizado no jogo, at aquele momento
		- Sexto parmetro do tipo "int" recebe o nvel da rvore
		- Stimo parmetro do tipo "int" recebe qual jogador realiza a jogada.
		- No h retorno.
	*****************************************************************************************************************************/
    void Imprimir(struct no *,int,int,int,int, int);

    /***************************************************************************************************************************
	  Jogada* verifica_jogada ();
	  Descrio:
		- Verifica as possveis jogadas diante de um determinado estado.
		- Primeiro parmentro do tipo "int" recebe qual jogador realiza a jogada.
		- Segundo parmentro do tipo "int" recebe a matriz das posies do tabuleiro.
        - Terceiro parmentro do tipo "int" recebe a posicao atual do rei branco.
        - Quarto parmentro do tipo "int" recebe a posicao atual do rei preto.
		- Quinto parmetro do tipo "int" recebe a quantidade de lances realizado no jogo, at aquele momento
		- Retorna uma lista ligadas com movimentos possveis.
	*****************************************************************************************************************************/
    struct reg* verifica_jogada (int,int M[][8],int,int,int);

    /***************************************************************************************************************************
	  Int minimax();
	  Descrio:
		- Avalia os ns da rvore de jogadas e retorna uma boa jogada a ser realizada
		- Primeiro parmentro do tipo "Arvore" recebe a arvore de jogadas.
		- Segundo parmentro do tipo "int" recebe a profundidade da rvore.
		- Terceiro parmentro do tipo "int" recebe o alfa da poda alfa-beta.
		- Quarto parmetro do tipo "int" recebe o bote da poda alfa-beta.
		- Retorna avaliao esttica da jogada escolhida.
	*****************************************************************************************************************************/
    float minimax(struct no *,int, float, float);

#endif /* _ARVORE_H_ */
