/*
    Programa: xeque.exe
    Descrio: Bibliotecas disponibilizadas na disciplina de PCII da Universidade Estadual de Mato Grosso do Sul e utilizada
    no projeto Xadrez.

    Programador: Eduardo Prates Albuquerque
    RGM: 21500
*/

#ifndef _CONSOLE
#define _CONSOLE

#include <windows.h>
/* -> Estrutura COORD:
      typedef struct _COORD{
              SHORT X;   -> inteiro curto
              SHORT Y;   -> inteiro curto
              } COORD;
   -> Tipos WORD, DWORD e LPSTR
*/

/****************************************************************************************************************/
/*                                  INCIO DAS DECLARAES DE DEFINES E CONSTANTES                              */
/****************************************************************************************************************/
#define F1 112
#define F2 113
#define F3 114
#define F4 115
#define F5 116
#define F6 117
#define F7 118
#define F8 119
#define F9 120
#define F10 121
#define F11 122
#define F12 123

#define ESC 27
#define ENTER 13
#define BACKSPACE 8
#define ESPACO 32
#define TAB 9

/* Enhanced keys */
#define INSERT 45
#define DEL 46
#define PAGE_UP 33
#define PAGE_DOWN 34
#define END 35
#define HOME 36
#define SETA_PARA_ESQUERDA 37
#define SETA_PARA_CIMA 38
#define SETA_PARA_DIREITA 39
#define SETA_PARA_BAIXO 40

/* Teclas de status */
#define SHIFT_ESQUERDO_PRESSIONADO 0x0200  /* (constante definida por mim = 0x200 = shift esquerdo pressionado) + (0x010 = shift pressionado) = 0x210. Valor definido com base em uma extenso dos valores contidos na tabela: http://msdn.microsoft.com/pt-br/library/windows/desktop/ms684166%28v=vs.85%29.aspx */
#define SHIFT_DIREITO_PRESSIONADO 0x0400  /* (constante definida por mim = 0x400 = shift direito pressionado) + (0x010 = shift pressionado) = 0x410. Valor definido com base em uma extenso dos valores contidos na tabela: http://msdn.microsoft.com/pt-br/library/windows/desktop/ms684166%28v=vs.85%29.aspx */


/* constantes definidas para identificar a liberao de um determinado boto do mouse */
#define FROM_LEFT_1ST_BUTTON_RELEASED FROM_LEFT_1ST_BUTTON_PRESSED
#define FROM_LEFT_2ND_BUTTON_RELEASED FROM_LEFT_2ND_BUTTON_PRESSED
#define FROM_LEFT_3RD_BUTTON_RELEASED FROM_LEFT_3RD_BUTTON_PRESSED
#define FROM_LEFT_4TH_BUTTON_RELEASED FROM_LEFT_4TH_BUTTON_PRESSED
#define RIGHTMOST_BUTTON_RELEASED RIGHTMOST_BUTTON_PRESSED

/* dar a opo ao programador de usar constantes em portugus, caso queira */
#define BOTAO_ESQUERDO_PRESSIONADO FROM_LEFT_1ST_BUTTON_PRESSED
#define BOTAO_ESQUERDO_LIBERADO FROM_LEFT_1ST_BUTTON_RELEASED
#define BOTAO_DIREITO_PRESSIONADO RIGHTMOST_BUTTON_RELEASED
#define BOTAO_DIREITO_LIBERADO RIGHTMOST_BUTTON_RELEASED

/* constantes utilizadas pela funo IdentificarEvento() para definir o nivel
   de detalhamento das informaes a serem coletadas de um evento */
#define MINIMO 1
#define MAXIMO 2

#define BIT_SHORT_MAIS_SIGNIFICATIVO 0x80  /* (0x80) na base hexadecimal = (128) na base 10 = (1000 0000) na base 2 */
#define TECLA_PRESSIONADA 0x80
/****************************************************************************************************************/
/*                                  FIM DAS DECLARAES DE DEFINES E CONSTANTES                              */
/****************************************************************************************************************/




/****************************************************************************************************************/
/*                                   DECLARAES DE ESTRUTURAS E ENUMERAES                                    */
/****************************************************************************************************************/

 /* estrutura para o armazenamento das cores da janela de console */
    typedef struct{
                  int CorTexto;      /* cor do texto da janela de prompt atual */
		          int CorFundo;      /* cor de fundo da janela de prompt atual */
                  }CORES;


 /* tipo STATUS contendo os valores para ligar e desligar o cursor do mouse da tela */
    typedef enum{DESLIGAR, LIGAR} STATUS;


 /* tipo VISIBILIDADE */
    typedef enum{INVISIVEL, VISIVEL} VISIBILIDADE;


 /* tipo que permite identificar que tipo de evento deve ser verificada a existncia no buffer de eventos */
    typedef enum{KEYBOARD_HIT, MOUSE_HIT, ALL} TYPE_HIT;


 /* Detalhes desta estrutura, ver link:
    http://msdn.microsoft.com/en-us/library/windows/desktop/ms684166%28v=vs.85%29.aspx */
    enum T_STATUS_TECLA{LIBERADA, PRESSIONADA};
    typedef struct T_TECLADO{
                            enum T_STATUS_TECLA status_tecla; /* LIBERADA ou PRESSIONADA */
		 				    DWORD codigo_tecla; /* cdigo numrico que identifica uma tecla independente do tipo do teclado */
                            DWORD status_teclas_controle; /* CAPSLOCK_ON, ENHANCED_KEY (insert, delete, home, end, pg up, pg downl,
						                                   setas de direo, etc), LEFT_ALT_PRESSED, LEFT_CTRL_PRESSED,
						                                   NUMLOCK_ON, RIGHT_ALT_PRESSED, RIGHT_CTRL_PRESSED, SCROLLLOCK_ON,
								 						   SHIFT_PRESSED */
                            } TECLADO;

 /* Detalhes desta estrutura, ver link:
    http://msdn.microsoft.com/en-us/library/windows/desktop/ms684239%28v=vs.85%29.aspx */
    typedef struct T_MOUSE{
                          DWORD botao_pressionado; /* FROM_LEFT_1ST_BUTTON_PRESSED, FROM_LEFT_2ND_BUTTON_PRESSED, FROM_LEFT_3RD_BUTTON_PRESSED,
                                                      FROM_LEFT_4TH_BUTTON_PRESSED, RIGHTMOST_BUTTON_PRESSED */
		 				  DWORD botao_clicou;      /* se o boto tiver sido liberado, aps ter sido pressionado */
                          COORD posicao; /* coordenadas (em forma de clulas de caracteres e no de pixels) do cursor do
						                    mouse em termos do tamanho (em linhas e colunas) do buffer
							 			    de console que estiver definido para a janela */
                          DWORD status_teclas_controle; /* CAPSLOCK_ON, ENHANCED_KEY (insert, delete, home, end, pg up, pg downl,
						                                   setas de direo, etc), LEFT_ALT_PRESSED, LEFT_CTRL_PRESSED,
						                                   NUMLOCK_ON, RIGHT_ALT_PRESSED, RIGHT_CTRL_PRESSED, SCROLLLOCK_ON,
							 							   SHIFT_PRESSED */
						  DWORD flags_qual_evento_mouse; /* DOUBLE_CLICK, MOUSE_HWHEELED, MOUSE_MOVED, MOUSE_WHEELED */
                          } MOUSE;

 /* Evento que pode ter acontecido e que  identificado pela funo Evento() */
    typedef struct T_EVENTO{
                           WORD tipo_evento; /* pode ser MOUSE_EVENT ou KEY_EVENT */
		  				   TECLADO teclado; /* caso tenha ocorrido um KEY_EVENT, ento esta estrutura ser prenchida
						                    com os dados correspondentes */
						   MOUSE mouse;    /* caso tenha ocorrido um MOUSE_EVENT, ento esta estrutura ser prenchida
						                    com os dados correspondentes */
                           }EVENTO;


/****************************************************************************************************************/
/*                                  FIM DAS DECLARAES DE ESTRUTURAS E ENUMERAES                             */
/****************************************************************************************************************/





/****************************************************************************************************************/
/*                                INCIO DAS DECLARAES DOS PROTTIPOS DAS FUNES                             */
/****************************************************************************************************************/

 /*****************************************************************************************************************
  Descrio: imprime a mensagem do tipo LPSTR (ponteiro para uma string) juntamente com o cdigo do ltimo erro
             ocorrido no programa obtido do fluxo de erro (stderr) e encerra o programa.
  Parmetros de entrada:
   -> Varivel do tipo LPSTR: uma mensagem a ser impressa na tela.
  Retorno da funo:
   -> Nenhum.
 *****************************************************************************************************************/
 void ErrorExit (LPSTR);




 /*****************************************************************************************************************
  Descrio: permitir e identificar com todos os detalhes, eventos originrios do teclado e/ou do mouse.
  Parmetros de entrada:
   -> Nenhum.
  Retorno da funo:
   -> Estrutura EVENTO:
      - Campo "tipo_evento" que  do tipo WORD (inteiro sem sinal de 2 bytes):
	      -> Valor MOUSE_EVENT: significa que o evento lido do buffer da janela de console  um evento originrio do
		                       MOUSE;
		  -> Valor KEY_EVENT: significa que o evento lido do buffer da janela de console  um evento originrio do
		                     TECLADO;
		  -> Observao 1: esta funo descarta os eventos FOCUS_EVENT, MENU_EVENT e WINDOW_BUFFER_SIZE_EVENT.
		                   Caso a identificao de algum desses eventos seja necessrio, a funo deve ser
			 			   estendida para que armazene neste campo o valor correspondente a um desses 3 eventos;
          -> Observao 2: o campo "tipo_evento"  um campo de bits. Isto , cada um dos valores indicados
		                   est associado a um bit do campo. Portanto, a identificao de qual valor est armazenado
                           no campo deve ser feito por meio dos operadores bit-a-bit.
						   Exemplo: EVENTO e;
						            e = Evento();
						            if (tipo_evento & MOUSE_EVENT) printf("Foi gerado um evento pelo mouse");
						            if (tipo_evento & KEY_EVENT) printf("Foi gerado um evento pelo teclado");
	  - Campo "teclado" que  uma estrutura do tipo TECLADO: se o campo "tipo_evento" contiver um valor KEY_EVENT
	                                                         ento esta estrutura conter todos os dados relativos
															 a esse evento. Caso contrrio, os valores contidos
															 nesta estrutura no tem sentido e, portanto, no devem
 															 ser utilizados.
	      -> Campo "status_tecla" do tipo enumerao: armazena se o evento do teclado foi o pressionamento
		                                              ou a liberao de uma tecla.
		     - Valor LIBERADA: evento corresponde  liberao do pressionamento de uma tecla;
			 - Valor PRESSIONADA: evento corresponde ao pressionamento de uma tecla.
		  -> Campo "codigo_tecla" do tipo DWORD: contem um valor inteiro nico independente do tipo do teclado para
		                                       cada uma das teclas do teclado.
		     - Observao:
			   Recomenda-se o uso das constantes definidas no incio deste arquivo (#defines) para
			   referenciar as teclas, ao invs da escrita no programa do prprio cdigo da tecla.
		  -> Campo "status_teclas_controle" do tipo DWORD (4 bytes): contem o status das diversas teclas de
		                                                             controle.
             - Valor CAPSLOCK_ON: a luz da tecla CAPSLOCK est ligada;
             - Valor ENHANCED_KEY: a tecla pressionada  uma tecla estendida (INSERT, DELETE, HOME, END, PG UP,
			                       PG DOWN, SETAS DE DIREO, etc.);
             - Valor LEFT_ALT_PRESSED: a tecla ALT da esquerda est pressionada;
             - Valor LEFT_CTRL_PRESSED: a tecla CTRL da esquerda est pressionada;
             - Valor NUMLOCK_ON: a luz da tecla NUMLOCK est ligada;
             - Valor RIGHT_ALT_PRESSED: a tecla ALT da direita est pressionada;
             - Valor RIGHT_CTRL_PRESSED: a tecla CTRL da direita est pressionada;
             - Valor SCROLLLOCK_ON: a luz da tecla SCROLLLOCK est ligada;
             - Valor SHIFT_PRESSED: uma, ou mesmo ambas, as teclas SHIFT esto pressionadas.
			 - Valor SHIFT_ESQUERDO_PRESSIONADO: se foi pressionada a tecla de SHIFT ESQUERDA;
			 - Valor SHIFT_DIREITO_PRESSIONADO: se foi pressionada a tecla de SHIFT DIREITO;
             -> Observao:
			    Este campo "status_teclas_controle"  um campo de bits. Isto , cada um dos valores indicados
		        est associado a um bit do campo. Portanto, a identificao de qual valor est armazenado
                no campo deve ser feito por meio dos operadores bit-a-bit.
				Exemplo: EVENTO e;
				         e = Evento();
				         if (e.teclado.status_teclas_controle & CAPSLOCK_ON) printf("A luz da tecla CAPSLOCK est acessa");
	  - Campo "mouse" que  uma estrutura do tipo MOUSE:
          -> Campo "botao_pressionado" do tipo DWORD:
             - Valor FROM_LEFT_1ST_BUTTON_PRESSED: primeiro boto da esquerda pressionado;
			 - Valor FROM_LEFT_2ND_BUTTON_PRESSED: segundo boto da esquerda pressionado;
			 - Valor RIGHTMOST_BUTTON_PRESSED: boto mais  direita pressionado.
             -> Observao 1:
			    Este campo "botao_pressionado"  um campo de bits. Isto , cada um dos valores indicados est
				associado a um bit do campo. Portanto, a identificao de qual valor est armazenado no campo deve
				ser feito por meio dos operadores bit-a-bit.
			    Exemplo: EVENTO e;
				         e = Evento();
				         if (e.mouse.botao_pressionado & RIGHTMOST_BUTTON_PRESSED)
						     printf("O boto da direita est pressionado");
             -> Observao 2:
			    Foram definidas, na seo "DEFINES e CONSTANTES", constantes em portugus, para os botes esquerdo
                e direito do mouse, que podem ser utilizadas ao invs dos valores anteriores para aumentar
				a legibilidade do cdigo:
				- BOTAO_ESQUERDO_PRESSIONADO
                - BOTAO_DIREITO_PRESSIONADO
             -> Observao 3:
			    Embora o "windows.h" colete e disponibilize informaes para mais dois botes que porventura o
				mouse tenha (constantes FROM_LEFT_3RD_BUTTON_PRESSED e FROM_LEFT_4TH_BUTTON_PRESSED, correspondentes
				ao terceiro e quarto botes da esquerda, respectivamente), esta funo ignora esses valores. Portanto,
				caso seja necessrio essa identificao, a funo deve ser estendida nesse aspecto.
		  -> Campo "botao_clicou" do tipo DWORD: contm se o evento foi a liberao de algum dos botes do mouse
		                                         que previamente estavam pressionados, definindo um clique do mouse.
             - Valor FROM_LEFT_1ST_BUTTON_RELEASED: primeiro boto da esquerda foi liberado (clicou);
			 - Valor FROM_LEFT_2ND_BUTTON_RELEASED: segundo boto da esquerda foi liberado (clicou);
			 - Valor RIGHTMOST_BUTTON_RELEASED: boto mais  direita foi liberado (clicou).
             -> Observao 1:
			    Este campo "botao_clicou"  um campo de bits. Isto , cada um dos valores indicados est
				associado a um bit do campo. Portanto, a identificao de qual valor est armazenado no campo deve
				ser feito por meio dos operadores bit-a-bit.
			    Exemplo: EVENTO e;
				         e = Evento();
				         if (e.mouse.botao_clicou & RIGHTMOST_BUTTON_RELEASED)
						     printf("O boto da direita clicou/foi liberado");
             -> Observao 2:
			    Foram definidas, na seo "DEFINES e CONSTANTES", constantes em portugus, para os botes esquerdo
                e direito do mouse, que podem ser utilizadas ao invs dos valores anteriores para aumentar
				a legibilidade do cdigo:
				- BOTAO_ESQUERDO_LIBERADO
                - BOTAO_DIREITO_LIBERADO
          -> Campo "posicao" que  uma estrutura do tipo COORD: guarda as coordenadas (x,y) na janela de console em
                                                                que ocorreu o evento do mouse. As coordenadas so
																dadas em posies de caracteres e no de pixels.
			 - Campo X: coluna da janela de console em que ocorreu o evento do mouse;
             - Campo Y: linha da janela de console em que ocorreu o evento do mouse.

		  -> Campo "status_teclas_controle" do tipo DWORD: contem o status das diversas teclas de controle, quando
		                                                   ocorreu o evento do mouse.
             - Valor CAPSLOCK_ON: a luz da tecla CAPSLOCK est ligada;
             - Valor ENHANCED_KEY: a tecla pressionada  uma tecla estendida (INSERT, DELETE, HOME, END, PG UP,
			                       PG DOWN, SETAS DE DIREO, etc.);
             - Valor LEFT_ALT_PRESSED: a tecla ALT da esquerda est pressionada;
             - Valor LEFT_CTRL_PRESSED: a tecla CTRL da esquerda est pressionada;
             - Valor NUMLOCK_ON: a luz da tecla NUMLOCK est ligada;
             - Valor RIGHT_ALT_PRESSED: a tecla ALT da direita est pressionada;
             - Valor RIGHT_CTRL_PRESSED: a tecla CTRL da direita est pressionada;
             - Valor SCROLLLOCK_ON: a luz da tecla SCROLLLOCK est ligada;
             - Valor SHIFT_PRESSED: uma, ou mesmo ambas, as teclas SHIFT esto pressionadas;
			 - Valor SHIFT_ESQUERDO_PRESSIONADO: se foi pressionada a tecla de SHIFT ESQUERDA;
			 - Valor SHIFT_DIREITO_PRESSIONADO: se foi pressionada a tecla de SHIFT DIREITO;
             -> Observao:
			    O campo "status_teclas_controle"  um campo de bits. Isto , cada um dos valores indicados
		        est associado a um bit do campo. Portanto, a identificao de qual valor est armazenado
                no campo deve ser feito por meio dos operadores bit-a-bit.
				Exemplo: EVENTO e;
				         e = Evento();
				         if (e.teclado.status_teclas_controle & CAPSLOCK_ON) printf("A luz da tecla CAPSLOCK est acessa");
		  -> Campo "flags_qual_evento_mouse" do tipo DWORD: identifica qual o tipo de evento do mouse que ocorreu.
             - Valor DOUBLE_CLICK: o segundo clique (pressionado de boto de um duplo-clique ocorreu.
               * Observao: o primeiro clique  retornado como um evento normal de pressionamento de boto.
             - Valor MOUSE_HWHEELED:  a roda de rolagem horizontal do mouse foi girada.
			   * Observao: se a HIGH WORD do campo "botao_pressionado" contiver um valor positivo, ento a roda foi
			                 girada para a direita. Caso contrrio, a roda foi girada para a esquerda.
			                 Exemplo de identificao: EVENTO e;
                                                       e = Evento();
                                                       if( (int)e.mouse.botao_pressionado > 0)
                                                          printf("A roda do mouse foi girada para a direita");
                                                       else
                                                          printf("A roda do mouse foi girada para a esquerda");
             - Valor MOUSE_MOVED: ocorreu uma mudana na posio do mouse;
             - Valor MOUSE_WHEELED: a roda de rolagem vertical do mouse foi girada.
			   * Observao: se a HIGH WORD do campo "botao_pressionado" contiver um valor positivo, ento a roda foi
			                 girada para a frente. Caso contrrio, a roda foi girada para trs.
			                 Exemplo de identificao: EVENTO e;
                                                       e = Evento();
                                                       if( (int)e.mouse.botao_pressionado > 0)
                                                          printf("A roda do mouse foi girada para frente");
                                                       else
                                                          printf("A roda do mouse foi girada para trs");
  Observaes:
   a) Deve ser dada preferncia ao uso desta funo ao invs da funo getch();
   b) Ver a declarao da estrutura EVENTO na seo anterior denominada "DECLARAES DE ESTRUTURAS E ENUMERAES".
   c) Ver a declarao da estrutura COORD na seo anterior denominada "DECLARAES DE ESTRUTURAS E ENUMERAES".
   d) Ver a declarao da estrutura TECLADO na seo anterior denominada "DECLARAES DE ESTRUTURAS E ENUMERAES";
   e) Ver a declarao da estrutura MOUSE na seo anterior denominada"DECLARAES DE ESTRUTURAS E ENUMERAES".
   f) O tipo WORD  um inteiro sem sinal longo de 2 bytes definido na "windows.h";
   g) O tipo DWORD  um inteiro sem sinal longo de 4 bytes definido na "windows.h";
   h) Os tipos comuns definidos na "windows.h" podem ser consultados em:
      http://msdn.microsoft.com/en-us/library/cc230309.aspx
 *****************************************************************************************************************/
 EVENTO Evento(void);





/*****************************************************************************************************************
  Descrio: obter as cores de fundo e do texto definidas para a janela de console
  Parmetros de entrada:
   -> Nenhum.
  Retorno da funo:
   -> Estrutura CORES:
      - Campo CorTexto: cor definida para a escrita de na janela de console.
	  - Campo CorFundo: cor definida para o fundo de qualquer caracteres escrito na
	                    janela de console

  Observao:
   -> Ver a declarao da estrutura CORES na seo anterior denominada
      "DECLARAES DE ESTRUTURAS E ENUMERAES".
 *****************************************************************************************************************/
 CORES getCoresJanela(void);




 /*****************************************************************************************************************
  Descrio: obter as coordenadas atuais da janela de console na tela.
  Parmetros de entrada:
   -> Nenhum.
  Retorno da funo:
   -> Estrutura COORD:
      - Campo X representa a linha do pixel da borda superior da janela de console.
      - Campo Y representa a coluna do pixel da borda esquerda da janela de console

  Observaes:
   a) A estrutura COORD  includa por meio do arquivo cabealho "windows.h". Contudo, a sua declarao
      pode ser consultada anteriormente neste mesmo arquivo, na linha em que  includo o arquivo
	  cabealho "windows.h".
   b) Em outras palavras, retorna as coodenadas (x,y) do pixel do canto superior
      esquerdo da janela de console.
 *****************************************************************************************************************/
 COORD getPosicaoJanela(void);




 /*****************************************************************************************************************
  Descrio: obter a resoluo do monitor em pixels:
  Parmetros de entrada:
   -> Nenhum.
  Retorno da funo:
   -> Estrutura COORD:
      - Campo X representa o nmero de pixels da largura
      - Campo Y representa o nmero de pixels da altura

  Observao:
   -> A estrutura COORD  includa por meio do arquivo cabealho "windows.h". Contudo, a sua declarao
      pode ser consultada anteriormente neste mesmo arquivo, na linha em que  includo o arquivo
	  cabealho "windows.h".
 *****************************************************************************************************************/
 COORD getResolucaoMonitor(void);




 /*****************************************************************************************************************
  Descrio: obter, em nmero de pixels, as dimenses horizontal e vertical da fonte da janela do console.
  Parmetros de entrada:
   -> Nenhum.
  Retorno da funo:
   -> Estrutura COORD:
      - Campo X representa a quantidade de pixels na horizontal ocupada por um caractere na janela do console.
      - Campo Y representa a quantidade de pixels na vertical ocupada por um caractere na janela do console.

  Observao:
   -> A estrutura COORD  includa por meio do arquivo cabealho "windows.h". Contudo, a sua declarao
      pode ser consultada anteriormente neste mesmo arquivo, na linha em que  includo o arquivo
	  cabealho "windows.h".
 *****************************************************************************************************************/
 COORD getTamanhoFonteConsole(void);




 /*****************************************************************************************************************
  Descrio: obter o ttulo da janela de console.
  Parmetros de entrada:
   -> Nenhum.
   Retorno da funo:
   -> Um ponteiro ponteiro para o endereo da string contendo o ttulo da janela, ou NULL se a janela
      no tiver um ttulo
 *****************************************************************************************************************/
 char *getTituloConsole(void);




 /*****************************************************************************************************************
  Descrio: obter se h algum evento (teclado, mouse, ou redimensionamento da janela) no buffer de eventos
             da janela de console.
  Parmetros de entrada:
   -> Enumerao TYPE_HIT:
      - Valor KEYBOARD_HIT: identificar somente os eventos originrios do teclado;
	  - Valor MOUSE_HIT: identificar somente os eventos originrios do mouse;
	  - Valor ALL: identificar qualquer evento presente no buffer de eventos da janela de console, sejam
	               oriundos do teclado, do mouse, ou do redimensionamento da janela.
  Retorno da funo:
   -> Um valor inteiro:
      - Valor 0: no h eventos do tipo solicitado no parmetro da funo no buffer de eventos da janela de
	             console;
	  - Valor 1: h um ou mais eventos do tipo solicitado no parmetro da funo no buffer de eventos
	             da janela de console.

  Observao:
   -> Ver a declarao da enumerao TYPE_HIT na seo anterior denominada
      "DECLARAES DE ESTRUTURAS E ENUMERAES".
 *****************************************************************************************************************/
 int hit(TYPE_HIT);




/*****************************************************************************************************************
  Descrio: apresentao detalhada de todas as informaes possveis de serem coletadas sobre um determinado evento
             ocorrido, cuja origem seja, ou o teclado, ou o mouse.
  Parmetros de entrada:
   -> Varivel do tipo da estrutura EVENTO:
      - Essa varivel deve conter os dados do evento lido anteriormente sobre o qual sero coletadas as informaes;
	  - Ver descrio do contedo dos campos da estrutura EVENTO na funo Evento();
	  - Exemplo de cdigo: EVENTO e;
	                       e = Evento();
						   IdentificarEvento(e, MAXIMO);
   -> Nmero inteiro que pode ser dois valores:
      - Valor 0: apresentar na tela o mnimo de informaes sobre o evento passado como parmetro;
	  - Valor 1: apresentar na tela o mximo de informaes (todas as possveis) sobre o evento passado como parmetro;
	  -> Observao:
	     Foram definidas duas constantes do tipo DEFINE para este parmetro, em substituio ao valores numricos 0 e 1.
		 Constantes MINIMO e MAXIMO, correspondente aos valores 0 e 1, respectivamente.
  Retorno da funo:
   -> Nenhum.

  Observao:
   -> Ver a declarao da estrutura EVENTO na seo anterior denominada "DECLARAES DE ESTRUTURAS E ENUMERAES".
 *****************************************************************************************************************/
 void IdentificarEvento(EVENTO, int);




 /*****************************************************************************************************************
  Descrio: obter as dimenses mximas possveis de uma janela na resoluo atual
             do monitor em pixels.
  Parmetros de entrada:
   -> Nenhum.
  Retorno da funo:
   -> Estrutura COORD:
     - Campo X representa o nmero de caracteres da largura
     - Campo Y representa o nmero de caracteres da altura

  Observaes:
   a) A estrutura COORD  includa por meio do arquivo cabealho "windows.h". Contudo, a sua declarao
      pode ser consultada anteriormente neste mesmo arquivo, na linha em que  includo o arquivo
	  cabealho "windows.h".
   b) Para poder utilizar esta funo  necessrio fazer uma correo no arquivo WINCON.H do gcc.
      Isso porque esta funo MaxDimensaoJanela(void) utiliza a funo da API do Windows
	  denominada GetCurrentConsoleFont() que somente est disponvel no Windows XP em diante e
 	  por isso o gcc no incluiu esse cabealho (embora exista sua implementao no ".lib" do gcc).
      - CORREO A SER FEITA:
	    Caso no exista, deve adicionar a seguinte linha no arquivo "wincon.h":
	         BOOL WINAPI GetCurrentConsoleFont(HANDLE, BOOL, PCONSOLE_FONT_INFO);
	  - ONDE DEVE SER ACRESCENTADA ESSA LINHA NO ARQUIVO "wincon.h":
        1) No arquivo "wincon.h" que est localizado dentro do diretrio "include" da sua instalao do gcc for Windows
	    2) A linha indicada deve ser adicionada dentro do define (caso no exista, crie ele)
		     #if (_WIN32_WINNT >= 0x0500)
		          BOOL WINAPI GetCurrentConsoleFont(HANDLE, BOOL, PCONSOLE_FONT_INFO);
		          ... -> mantenha tudo o que j existir anteriormente definido dentro do define
		     #endif
 *****************************************************************************************************************/
 COORD MaxDimensaoJanela(void);




 /*****************************************************************************************************************
  Descrio: ligar ou desligar o cursor piscante na janela de console.
  Parmetros de entrada:
   -> Uma string contendo o novo ttulo da janela de console.
  Retorno da funo:
   -> Enumerao STATUS:
      - Valor DESLIGAR: desligar o cursor piscante na janela de console;
	  - Valor LIGAR: ligar o cursos piscante na janela de console.

  Observao:
   -> Ver a declarao da enumerao STATUS na seo anterior denominada
      "DECLARAES DE ESTRUTURAS E ENUMERAES".
 *****************************************************************************************************************/
 void setCursorStatus(STATUS);




 /*****************************************************************************************************************
  Descrio: funo que permite redimensionar a janela de console.
  Parmetros de entrada:
   -> Estrutura COORD:
      - Campo X representa o nmero de caracteres da nova largura da janela
      - Campo Y representa o nmero de caracteres da nova altura da janela
  Retorno da funo:
   -> Nenhum.

  Observao:
   - Recomenda-se que obtenha os valores mximos possveis para o redimensionamento
     da janela do console por meio de uma chamada  funo MaxDimensaoJanela(). Desse
     modo, conseguir garantir que os valores passados como parmetros para esta funo
     no ultrapassar os valores mximos possveis de redimensionamento da janela do
     console.
 *****************************************************************************************************************/
 void setDimensaoJanela(int largura, int altura);




 /*****************************************************************************************************************
  Descrio: esconder ou tornar visvel a Barra de Tarefas da rea de trabalho do Windows.
  Parmetros de entrada:
   -> Enumerao VISIBILIDADE:
      - Valor VISIVEL: tornar visvel a Barra de Tarefas;
      - Valor INVISIVEL: esconder a Barra de Tarefas.
  Retorno da funo:
   -> Nenhum.

  Observao:
   -> Ver a declarao da enumerao VISIBILIDADE na seo anterior denominada
      "DECLARAES DE ESTRUTURAS E ENUMERAES".
 *****************************************************************************************************************/
 void setEstadoBarraTarefas(VISIBILIDADE);




/*****************************************************************************************************************
  Descrio: posicionar a janela em uma determinada posio na tela.
  Parmetros de entrada:
   -> Estrutura COORD:
      - Campo X representa a linha do pixel da borda superior da janela de console.
      - Campo Y representa a coluna do pixel da borda esquerda da janela de console
  Retorno da funo:
   -> Nenhum.
 *****************************************************************************************************************/
 void setPosicaoJanela(int top, int left);




 /*****************************************************************************************************************
  Descrio: definir um titulo para a janela de console.
  Parmetros de entrada:
   -> Uma string contendo o novo ttulo da janela de console.
  Retorno da funo:
   -> Nenhum.
 *****************************************************************************************************************/
 void setTituloConsole(char *);




 /*****************************************************************************************************************
  Descrio: obter o tamanho da janela de console atual em quantidade de caracteres.
  Parmetros de entrada:
   -> Nenhum.
  Retorno da funo:
   -> Estrutura COORD:
      - Campo X representa a largura da janela de console em nmero de caracteres
      - Campo Y representa a altura da janela de console em nmero de caracteres

  Observao:
   -> A estrutura COORD  includa por meio do arquivo cabealho "windows.h". Contudo, a sua declarao
      pode ser consultada anteriormente neste mesmo arquivo, na linha em que  includo o arquivo
	  cabealho "windows.h".
 *****************************************************************************************************************/
 COORD tamanhoJanelaConsole(void);




/****************************************************************************************************************/
/*                                  FIM DAS DECLARAES DOS PROTTIPOS DAS FUNES                              */
/****************************************************************************************************************/

#endif
